import { app } from '@/app.ts';
import { MicroAppUploadModel } from './models.ts';
import { appPathCheck, installApp } from './module/install-app.ts';
import { manager } from './manager-app.ts';
import { selfRestart } from '@/modules/self-restart.ts';

// 应用上传到 应用管理 的平台
app
  .route({
    path: 'micro-app',
    key: 'upload',
    middleware: ['auth'],
    description: 'Upload micro app in server',
    isDebug: true,
  })
  .define(async (ctx) => {
    const { files, collection } = ctx.query?.data;
    const { uid, username } = ctx.state.tokenUser;
    const file = files[0];
    console.log('File', files);
    const { path, name, hash, size } = file;
    const tags = [];
    if (collection?.tags) {
      tags.push(...collection.tags);
    }
    if (!name) {
      ctx.throw(400, 'Invalid file');
    }
    let microApp = await MicroAppUploadModel.findOne({
      where: { title: name, uid },
    });
    if (microApp) {
      await MicroAppUploadModel.update(
        {
          type: 'micro-app',
          tags,
          data: {
            ...microApp.data,
            file: {
              path,
              size,
              name,
              hash,
            },
            collection,
          },
        },
        { where: { title: name, uid } },
      );
      microApp = await MicroAppUploadModel.findOne({
        where: { title: name, uid },
      });
      console.log('Update micro app', microApp.id);
    } else {
      microApp = await MicroAppUploadModel.create({
        title: name,
        description: collection?.readme || '',
        type: 'micro-app',
        tags: tags,
        data: {
          file: {
            path,
            size,
            name,
            hash,
          },
          collection,
        },
        uid,
        share: false,
        uname: username,
      });
    }

    ctx.body = microApp;
  })
  .addTo(app);

// curl http://localhost:4002/api/router?path=micro-app&key=deploy
// 把对应的应用安装到系统的apps目录下，并解压，然后把配置项写入数据库配置
// key 是应用的唯一标识，和package.json中的key一致，绑定关系
// 安装就就一定存在状态。这个时候需要判断这个应用的依赖关系，这个应用的开启状态。
// 修改app的类型，这些模块是存在添加，不存在则删除。
app
  .route({
    path: 'micro-app',
    key: 'deploy',
    description: 'Deploy micro app in server',
  })
  .define(async (ctx) => {
    const { id, key, force } = ctx.query?.data;
    // const id = '10f03411-85fc-4d37-a4d3-e32b15566a6c';
    // const key = 'envision-cli';
    // const id = '7c54a6de-9171-4093-926d-67a035042c6c';
    // const key = 'mark';
    if (!id) {
      ctx.throw(400, 'Invalid id');
    }
    const microApp = await MicroAppUploadModel.findByPk(id);
    const { file } = microApp.data || {};
    const path = file?.path;
    if (!path) {
      ctx.throw(404, 'Invalid path');
    }
    console.log('path', path);
    const check = await appPathCheck({ key });
    let appType: string;
    if (check) {
      if (!force) {
        ctx.throw(400, 'App already exists, please remove it first');
      } else {
        const app = manager.getAppShowInfo(key);
        appType = app?.type;

        await manager.removeApp(key);
      }
    }
    const installAppData = await installApp({ path, key });
    await manager.add(installAppData.showAppInfo);
    ctx.body = installAppData;
    if (appType === 'system-app') {
      // 如果是system-app(主进程运行），就重启服务
      setTimeout(() => selfRestart(), 3000);
    }
  })
  .addTo(app);

//  curl http://localhost:4002/api/router?path=micro-app&key=unload
app
  .route({
    path: 'micro-app',
    key: 'unload',
    description: 'Unload micro app in server',
  })
  .define(async (ctx) => {
    const { key } = ctx.query?.data;
    // const key = 'mark';
    const main = manager.getAppShowInfo(key);
    if (!main) {
      ctx.throw(400, 'Invalid app');
    }
    manager.removeApp(key);
    ctx.body = main;
  })
  .addTo(app);
