import { Resource } from '../models/index.ts';
import { minioClient, bucketName } from '../../../modules/minio.ts';

type MinioRes = {
  etag?: string; // 文件的etag, 用于后续的文件下载
  versionId?: string;
};

type PublishOptions = {
  name: string;
  version: string;
  code: string;
};

export const publishJsCode = async ({ name, version, code }: PublishOptions) => {
  // publish to minio
  const codeBuffer = Buffer.from(code);
  const codePath = `${name}/${version}/index.js`;
  try {
    const res = await minioClient.putObject(bucketName, codePath, codeBuffer, codeBuffer.length, {
      'Content-Type': 'application/javascript',
      'Cache-Control': 'max-age=31536000, immutable',
    });
    return {
      code: 200,
      data: { ...res, path: codePath },
    };
  } catch (e) {
    console.error('publish error', e.message);
    return {
      code: 500,
      message: e.message,
    };
  }
};
